package mactrans.system2;

/**
 * System Configuration (a)
 * 
 * These methods must be implemented in order to make exchanges
 * between the UI and the system work.
 * 
 * 
 * @author Jan Kienecker (3kieneck@informatik.uni-hamburg.de)
 * @version $Revision: 1.2 $
 * 
 * <pre>
 * $Log: MTSystemLayout1.java,v $
 * Revision 1.2  2005/05/30 11:42:30  jkieneck
 * + MTSL will scan for MTPs automatically
 *
 * Revision 1.1  2005/05/25 12:54:41  jkieneck
 * + added to repository
 *
 * </pre>
 */

import java.util.Iterator;

public class MTSystemLayout1 implements MachineTranslationSystemInterface {
	private MachineTranslationClass dup;
	
    public MTSystemLayout1() {
        // create the core first of all
        new MachineTranslationCore();
        
        // create a local instance of the MTC
        dup = new MachineTranslationClass();
        MachineTranslationCore.addModule( dup, "Worker" );

  	    // create the processor module; this is a stupid workaround, as this call
        // should usually be contained in the construct part. But since construct
        // is called *after* the initialization of the UI, the getAlgorithms must
        // return existing processors *before* the construct call is made. Maybe
        // this will be fixed in the future, is not too serious, though..
	    MachineTranslationCore.addModule( new Matching(), "Matching" );
  	    MachineTranslationCore.addModule( new TestProcessor(), "TestProcessor" );
    }

    /**
	 * Init the system
	 *
	 */
	public void construct( MachineTranslationGUInterface mts ) {
  	    MachineTranslationCore.mtGUI = mts;
  	    
  	    // create input and output modules
  	    MachineTranslationCore.addModule( new IOBase(), "input" );
  	    MachineTranslationCore.addModule( new IOBase(), "output" );
  	    
  	    // create the results module
  	    MachineTranslationCore.addModule( new Results(), "results" );
  	    
  	    dup.debug( "Initialization successful. The following modules are loaded: " );
  	    Iterator it = MachineTranslationCore.moduleIterator();
  	    while ( it.hasNext() ) {
  	        AbstractMachineTranslationClass tmp = (AbstractMachineTranslationClass) it.next();
  	        dup.debug( " > " + tmp.getModuleName() );
  	    }
	}
	
	/**
	 * Destroy the system
	 *
	 */
	public void destruct() {
	}
	
	/**
	 * A list of supported source languages
	 * 
	 * @return  a String Array as {EN,DE...}
	 */
	public String[] getSourceLanguages() {
		String[] result = new String[3];
		result[0] = "CZ";
		result[1] = "EN";
		result[2] = "DE";
		return result;
	}

	/**
	 * A list of supported target languages
	 * 
	 * @return  a String Array as {EN,DE...}
	 */
	public String[] getTargetLanguages() {
		String[] result = new String[3];
		result[0] = "CZ";
		result[1] = "EN";
		result[2] = "DE";
		return result;
	}
	
	/**
	 * A list of supported algorithms
	 * 
	 * @return a String Array such as {EditDistance...}
	 */
	public String[] getAlgorithms() {
	    int i = 0;
	    Iterator it = MachineTranslationCore.moduleIterator();
	    while ( it.hasNext() ) {
	        if ( it.next() instanceof MachineTranslationProcessor ) {
	            i++;
	        }
	    }
		String[] result = new String[i];
	    Iterator it2 = MachineTranslationCore.moduleIterator();
	    i = 0;
	    while ( it2.hasNext() ) {
	        AbstractMachineTranslationClass tmp = (AbstractMachineTranslationClass) it2.next();
	        if ( tmp instanceof MachineTranslationProcessor ) {
	            result[ i ] = tmp.getModuleName();
	            i++;
	        }
	    }
	    return result;
	}
	
	/**
	 * Tell the system which language is the input language
	 * 
	 * @param input a String such as EN,DE
	 */
	public void setSourceLanguage( String input ) {
	    dup.input().setLanguage( input );
	}
	
	/**
	 * Tell the system which language is the output language
	 * 
	 * @param input a String such as EN,DE
	 */
	public void setTargetLanguage( String input ) {
	    dup.output().setLanguage( input );
	}
	
	/**
	 * Tell the system which algol to use
	 * 
	 * @param input a String such as EditDistance
	 */
	public void setAlgorithm( String input ) {
	    // TODO apply method for setting algorithm
	}
	
	/**
	 * Tell the system what's the source of everything...
	 * 
	 * @param input the sentence entered by the user
	 */
	public void setInputPhrase( String input ) {
	    dup.input().setSentence( input );
	}
	
	/**
	 * Make the system translate based on the given environment
	 *
	 */
	public void translate() {
	    // TODO dup.callProcessor( ... )
	    dup.callProcessor( "Matching" );
	}
	
	/**
	 * Return the result of all the translation work
	 * 
	 * @return a String (the result)
	 */
	public String getOutputPhrase() {
	    return dup.output().getSentence();
	}
}
