package mactrans.system2;

// import mactrans.database.*;

/**
 * @author Jan Kienecker (3kieneck@informatik.uni-hamburg.de)
 * @version $Revision: 1.2 $
 * 
 * <pre>
 * $Log: MachineTranslationClass.java,v $
 * Revision 1.2  2005/05/30 11:41:54  jkieneck
 * + added database import
 *
 * Revision 1.1  2005/05/25 12:54:41  jkieneck
 * + added to repository
 *
 * </pre>
 */

public class MachineTranslationClass extends AbstractMachineTranslationClass {
  /*** Environment and processors ***/
  
  /**
   * Access the input module
   * 
   * @return the assigned instance
   */
  public IOBase input() {
    return (IOBase) MachineTranslationCore.get( "input" );
  }
  
  /**
   * Access the output module
   * 
   * @return the assigned instance
   */
  public IOBase output() {
    return (IOBase) MachineTranslationCore.get( "output" );
  }
  
  /**
   * Access the GUI methods
   * 
   * @return the assigned GUI
   */
  public MachineTranslationGUInterface mtGUI() {
    return MachineTranslationCore.getGUI();
  }
  
  /**
   * Access the results module
   * 
   * @return the assigned instance
   */
  public Results results() {
    return (Results) MachineTranslationCore.get( "results" );
  }
  
  /**
   * Call a MT processor (algorithm)
   * 
   * @return true, if the called MTP did not return null
   */
  public boolean callProcessor( MachineTranslationProcessor mtp ) {
    Object result = mtp.proceed();
    results().storeResultOf( mtp, result );
    return ( result != null );
  }
  
  /**
   * Call a MT processor by its public name
   * 
   * @param mtpName the public name of the MTP
   * @return true, if the called MTP did not return null
   */
  public boolean callProcessor( String mtpName ) {
    return callProcessor( MachineTranslationCore.getMTP( mtpName ) );
  }
  
  /**
   * Find the returned object/data of a given MTP
   * 
   * @param mtp the MTP to look the results for
   * @return the data produced by the MTP or null, if none was found
   */
  public Object getResultOf( MachineTranslationProcessor mtp ) {
    return results().getResultOf( mtp );
  }
  
  /**
   * Find the returned object/data of a given MTP by its public name
   * @param mtpName the public name of the MTP
   * @return the data produced by the MTP or null, if none was found
   */
  public Object getResultOf( String mtpName ) {
    return results().getResultOf( MachineTranslationCore.getMTP( mtpName ) );
  }
  
  /*** Verbosity (debugging/errors) ***/
  
  /**
   * Debug output
   * 
   * @param message the text to output
   */
  public void debug( String message ) {
    mtGUI().debug( getModuleName(), message );
  }

  /**
   * Warn output
   * 
   * @param message the text to output
   */
  public void warn( String message ) {
    mtGUI().warn( getModuleName(), message );
  }

  /**
   * Error output
   * 
   * @param message the text to output
   */
  public void error( String message ) {
    mtGUI().error( getModuleName(), message );
  }

  /**
   * Fatal error output
   * 
   * @param message the text to output
   */
  public void fatalError( String message ) {
    mtGUI().fatalError( getModuleName(), message );
  }
  

  /** mt
   * Output of translated sentence
   */
  public void writeOutput( String sentence ) {
    mtGUI().writeOutput( sentence );
  }
  
  /*** Translation memory and dictionaries ***/
  
  /**
   * Access a translation memory (language)
   * 
   * @param language the language of the translation memory (like EN, CZ, etc)
   * @return access to the translation memory or null, if the tm was not found 
   */
/*  public TranslationMemory translationMemory( String language ) {
    return (TranslationMemory) MachineTranslationCore.get( "tm" + language );
  }

  /**
   * Access a translation dictionary (language)
   * 
   * @param language the language of the translation dictionary (like EN, CZ, etc)
   * @return access to the translation dictionary or null, if the td was not found 
   */
/*  public TranslationDictionary translationDictionary( String language ) {
    return (TranslationDictionary) MachineTranslationCore.get( "td" + language );
  }
  
  /**
   * Access a dictionary (language)
   * 
   * @param language the language of the dictionary (like EN, CZ, etc)
   * @return access to the dictionary or null, if the dict was not found 
   */
/*  public Dictionary dictionary( String language ) {
    return (Dictionary) MachineTranslationCore.get( "dict" + language );  
  } 
*/
}
