package mactrans.system2;

/**
 * This is a static class which will store references to required modules.
 * 
 * @author Jan Kienecker (3kieneck@informatik.uni-hamburg.de)
 * @version $Revision: 1.2 $
 * 
 * <pre>
 * $Log: MachineTranslationCore.java,v $
 * Revision 1.2  2005/05/30 11:42:06  jkieneck
 * + added iterator support
 *
 * Revision 1.1  2005/05/25 12:54:41  jkieneck
 * + added to repository
 *
 * </pre>
 */

import java.util.HashMap;
import java.util.Iterator;

public final class MachineTranslationCore {
  private static HashMap moduleMap;
  public static MachineTranslationGUInterface mtGUI;
  
  public MachineTranslationCore() {
    moduleMap = new HashMap();
  }
  
  /**
   * Adds a module using the internal name
   * 
   * @param mtc an abstract MTC
   * @return false, if a module by that name exists, true otherwise
   */
  public static boolean addModule( AbstractMachineTranslationClass mtc ) {
    boolean result = true;
    if ( moduleMap.containsKey( mtc.getModuleName() ) ) {
      result = false;
    }
    moduleMap.put( mtc.getModuleName(), mtc );
    return result;
  }
  
  /**
   * Adds a module using a given name, overwrites the internal name of the module
   * 
   * @param mtc an abstract MTC
   * @param name the name of the module which it is publicly known as
   * @return false, if a module by that name exists, true otherwise
   */
  public static boolean addModule( AbstractMachineTranslationClass mtc, String name ) {
    mtc.setModuleName( name );
    return addModule( mtc );
  }
  
  /**
   * Access an MTC object as abstract
   * 
   * @param name the public name of the module
   * @return a found module or null
   */
  public static AbstractMachineTranslationClass get( String name ) {
    return (AbstractMachineTranslationClass) moduleMap.get( name );
  }
  
  /**
   * Access an MTC object as MTClass
   * 
   * @param name the public name of the module
   * @return a found module or null otherwise
   */
  public static MachineTranslationClass getMTC( String name ) {
    return (MachineTranslationClass) moduleMap.get( name );
  }
  
  /**
   * Access an MTC object as MTService
   * 
   * @param name the public name of the module
   * @return a found module or null otherwise
   */
  public static MachineTranslationService getMTS( String name ) {
    return (MachineTranslationService) moduleMap.get( name );
  }
  
  /**
   * Access an MTC object as MTProcessor
   * 
   * @param name the public name of the module
   * @return a found module or null otherwise
   */
  public static MachineTranslationProcessor getMTP( String name ) {
    return (MachineTranslationProcessor) moduleMap.get( name );
  }
  
  /**
   * Access the methods provided by the GUI
   * 
   * @return access to the GUI interface (mainly for output control)
   */
  public static MachineTranslationGUInterface getGUI() {
    return mtGUI;
  }
  
  /**
   * Creates an iterator on the modules
   * 
   * @return an iterator to walk through the values
   */
  public static Iterator moduleIterator() {
      return moduleMap.values().iterator();
  }
}