use Tie::RefHash;
use warnings;
use strict;

#finds chunks in the translation memory for each word in the input sentence,
#restricting the chunks to those present in the input sentence
sub sentence_enhance_with_matching_chunks
{
  my ($input_sentence, $input_lang_morphology, $input_lang_mem, $debug) = @_;
  foreach my $word (@{ $input_sentence->{sequence} })
    {
      print "\n\nWORD OF INPUT SENTENCE: $word->{token}\n" if $debug;
      foreach my $lemma (@{ morpho_analyse_get_lemmas($input_lang_morphology, $word->{token}) })
	{
	  print "LEMMA: $lemma\n" if $debug;
	  foreach my $occurence (@{ memory_get_occurences($input_lang_mem, $lemma) })
	    {
	      print "OCCURENCES IN TM: sentence_id $occurence->{sentence_id}\t chunk_number $occurence->{chunk_number}\n" if $debug;
	      my $positions_of_chunks_words = sentence_chunk_is_in($input_sentence, $input_lang_mem, $occurence, $input_lang_morphology, 1);
	      push @{ $word->{chunks} }, { occurence => $occurence, positions => $positions_of_chunks_words } if $positions_of_chunks_words
	    }
	}
    }
}

sub sentence_chunk_is_in
{
  my ($sentence, $memory, $chunk, $morphology, $debug) = @_;
  my $chunks_sentence = memory_get_sentence($memory, $chunk);
  my @result;

  do {
      print "CHUNK CANDIDATE:\n";
      foreach my $chunks_element (@{ memory_get_chunk_elements($memory, $chunk) })
	{
	  print "\telement $chunks_element: ", $chunks_sentence->{words}[$chunks_element], "\n"
	}
  } if $debug;

  CHUNK_ELEMENT: foreach my $chunks_element (@{ memory_get_chunk_elements($memory, $chunk) })
  {
    foreach my $lemma (@{ morpho_analyse_get_lemmas($morphology, $chunks_sentence->{words}[$chunks_element]) })
    {
      if (exists $sentence->{types}{$lemma}) { push @result, $sentence->{types}{$lemma} ; next CHUNK_ELEMENT }
    }
    print "\tFAILED\n" if $debug;
    return undef
  }
  print "\tPRESENT\n" if $debug;
  \@result;
}

sub sentence_print_matching_chunks
{#use on a sentence enhanced with matching chunks
  my($input_sentence, $input_lang_mem, $target_lang_mem) = @_;
  print "\nREVIEW:\n", '-'x7, "\n";
  foreach my $word (@{ $input_sentence->{sequence} })
    {
      print "WORD $word->{token} was found in these chunks:\n";
      my $counter = 0;
      foreach my $occurence (@{ $word->{chunks} })
	{
	  $counter++;
	  my $chunk_sentence = memory_get_sentence($input_lang_mem, $occurence->{occurence});
	  print "\t$counter: with words ", join(', ', map { $chunk_sentence->{words}->[$_] } @{ memory_get_chunk_elements($input_lang_mem, $occurence->{occurence}) }), ' IN TARGET LANG ', join(' ', @{ transfer($target_lang_mem, $occurence->{occurence}) } ), "\n"
	}
    }
}

sub sentence_enhance_with_shared_chunks
{#$input_sentence
  my (%chunks, %word_sequences, $counter, $debug); $counter = 0;
  tie %chunks, 'Tie::RefHash';
  foreach my $word (@{ $_[0]->{sequence} })
    {
      foreach my $occurence (@{ $word->{chunks} })
	{
	  my $occ_occ = $occurence->{occurence};
	  print 'SENTENCE_ENHANCE_WITH_SHARED_CHUNKS: ', memory_chunk_occurence_to_str($occ_occ), "\n";
	  foreach my $chunk_realization (@{ expand_2D_array_to_paths($occurence->{positions}) })
	    {
	      my $positions_key = join(',', @{ $chunk_realization });
	      if (exists $chunks{$occ_occ} and exists $chunks{$occ_occ}->{$positions_key})
		{
		  print "already seen\n"
		}
	      else
		{
		  $chunks{$occ_occ}->{$positions_key} = $occurence->{positions};
		  print "\tnewly seen positions_key $positions_key at occurence (", memory_chunk_occurence_to_str($occ_occ), ")\n"
		}
	    }
	}
      $counter++
    }
  $_[0]->{chunks} = \%chunks;
}

sub sequence_is_growing
{
  my $element = shift;
  my ($min, $max) = ($element, $element);

  while (my $next_element = shift)
  {
    if ($next_element > $element)
      {
	$element = $next_element;
	if ($element < $min) { $min = $element }
	if ($element > $max) { $max = $element }
      }
    else { return undef }
  }
  $max - $min
}

sub element
{
  foreach my $element (@{ $_[1] }) { return 1 if $_[0] == $element }
  undef
}

sub sentence_filter_shared_chunks_cut_delete_realizations
{
  for (my $i = 0; $i < @{ $_[0] };)
  {
    if (element($_[1], $_[0]->[$i]{key_array})) { delete $_[0]->[$i] }
    else { $i++ }
  }
}

sub sentence_filter_shared_chunks_cut
{
  my @realizations = sort { $a->{dist} <=> $b->{dist} } @{ $_[0] };
  while (my $realization = shift @realizations)
    {
      foreach my $word_index (@{ $realization->{key_array} })
      {
        sentence_filter_shared_chunks_cut_delete_realizations(\@realizations, $word_index)
      }
    }
}

sub sentence_filter_shared_chunks
{#$input_sentence
  while (my ($chunk_occ, $realizations) = each %{ $_[0]->{chunks} })
    {
      my @realizations_to_cut;
      while (my ($positions_key, $chunk_realizations) = each %{ $realizations })
	{
	  my @positions = split(/,/, $positions_key);
	  my $dist = sequence_is_growing(@positions);
	  if (not defined $dist) { delete $realizations->{$positions_key} }
	  else { push @realizations_to_cut, { dist => $dist, key => $positions_key, key_array => \@positions }; }
	}
      sentence_filter_shared_chunks_cut(\@realizations_to_cut, $realizations)
    }
}


sub sentence_print_chunks
#use on a sentence enhanced with shared chunks
{#$input_sentence, memory, $target_lang_mem
  my $target_lang_mem = $_[2];
  my $tm_occ_index = 0;
  while (my ($tm_chunk_occ_ref, $realizations) = each %{ $_[0]->{chunks} })
  {
    print "sent $tm_chunk_occ_ref->{sentence_id}, chunk $tm_chunk_occ_ref->{chunk_number} = ", memory_chunk_get_words($_[1], $tm_chunk_occ_ref), ' ~ ', join(' ', @{ transfer($target_lang_mem, $tm_chunk_occ_ref) }) , "\n";
    while (my ($positions_key, $chunk_realizations) = each %{ $realizations })
    {
      print "\tis realized on these words of InS: $positions_key (", join(' ', map { $_[0]->{sequence}[$_]{token} } split(/,/, $positions_key)) ,")\n";
      foreach my $chunk_element_realization (@{ $chunk_realizations })
      {
        print "\t\t", join('-', @{ $chunk_element_realization }), "\n"
      }
    }
  }
}

sub expand_2D_array_to_paths
{
  my @result = map { [$_] } @{ $_[0]->[0] };
  foreach my $chunk_part (@{ $_[0] }[ 1..$#{$_[0]} ])
    {
      @result = map { my @res; foreach my $item (@{ $chunk_part }) { push @res, [@{$_}, $item] } @res } @result
    }
  return \@result
}


1
