#!/usr/bin/perl

use warnings;
use strict;
use lib '/home/tynovsky/data';

use mbase_cz;
use mbase_de;
use mbase_en;

use morpho;
use memory;
use sentence;
use matching;
use transfer;

my $cz_morpho = morpho_build(czech_morpho_base_get());
my $de_morpho = morpho_build(german_morpho_base_get());
my $en_morpho = morpho_build(english_morpho_add_ing_form(english_morpho_base_get()));

my ($cz_mem, $de_mem, $en_mem) = read_translation_memory('/home/veronika/ebmt/memory', $cz_morpho, $de_morpho, $en_morpho);

my $lang_data = {
    cz => { morphology => $cz_morpho, mem => $cz_mem },
    de => { morphology => $de_morpho, mem => $de_mem },
    en => { morphology => $en_morpho, mem => $en_mem }
};


my $input_sentence = 'V zemích s odlišnými klimatickými podmínkami v motoru v katalyzátoru země';
 $input_sentence = 'Neměli byste používat žádné vozidlo';
 $input_sentence = 'You should not use the vehicle';
 $input_sentence = 'Použijte bezolovnatý benzin pro vozidla s katalyzátorem .';

my ($input_lang, $output_lang) = ('cz', 'en');


$input_sentence = sentence_create_structure($input_sentence, $lang_data->{$input_lang}{morphology});

sentence_enhance_with_matching_chunks($input_sentence, $lang_data->{$input_lang}{morphology}, $lang_data->{$input_lang}{mem}, 1);
sentence_print_matching_chunks($input_sentence, $lang_data->{$input_lang}{mem}, $lang_data->{$output_lang}{mem});

sentence_enhance_with_shared_chunks($input_sentence);
sentence_filter_shared_chunks($input_sentence);
sentence_print_chunks($input_sentence, $lang_data->{$input_lang}{mem}, $lang_data->{$output_lang}{mem});

print_coverage(find_coverage($input_sentence, $lang_data->{$input_lang}{mem}), $lang_data->{$output_lang}{mem});



while (my ($word, $occ) = each %{ $input_sentence->{types} })
{
  print $word, ": ", join(', ', @{ $occ }), "\n"
}

