use warnings;
use strict;

use Scalar::Util qw(looks_like_number);

my $log;
open($log, '>memory.log');

sub read_translation_memory	#loads the translation memory into data structures from a file
{
  my ($filename, $cz_morpho, $de_morpho, $en_morpho) = @_;
  my ($fh, @rec, @cz_sentences, @de_sentences, @en_sentences, %cz_map, %de_map, %en_map); $#rec=10;
  open($fh, $filename) or die "Can't open file $filename: $!";

	#loads a multi-lined record and parses it into data structures, checking for data integrity
  while ((@rec = (scalar <$fh>, scalar <$fh>, scalar <$fh>, scalar <$fh>, scalar <$fh>, scalar <$fh>, scalar <$fh>, scalar <$fh>, scalar <$fh>, scalar <$fh>, scalar <$fh>)), $rec[10])
  {
    chomp @rec;
    my ($id, @czse, $czch, $cztr, @dese, $dech, $detr, @ense, $ench, $entr);
    if ($rec[0] =~ /^SEID=([0-9]+)$/) { $id = $1 + 0 } else { die "Error on reading SEID in translation memory" }
    if ($rec[1] =~ /^CZSE=(.+)$/) { @czse = split(/ /, $1) } else { die "Error on reading CZSE (SEID=$id) in translation memory" }
    if ($rec[2] =~ /^DESE=(.+)$/) { @dese = split(/ /, $1) } else { die "Error on reading DESE (SEID=$id) in translation memory" }
    if ($rec[3] =~ /^ENSE=(.+)$/) { @ense = split(/ /, $1) } else { die "Error >$1< on reading ENSE (SEID=$id) in translation memory" }
    if ($rec[4] =~ /^CZCH=(.+)$/) { $czch = parse_chunks($1, $id, scalar @czse) } else { die "Error on reading CZCH (SEID=$id) in translation memory" }
    if ($rec[5] =~ /^DECH=(.+)$/) { $dech = parse_chunks($1, $id, scalar @dese) } else { die "Error on reading DECH (SEID=$id) in translation memory" }
    if ($rec[6] =~ /^ENCH=(.+)$/) { $ench = parse_chunks($1, $id, scalar @ense) } else { die "Error on reading ENCH (SEID=$id) in translation memory" }
    if ($rec[7] =~ /^CZTR=(.*)$/) {  } else { die "Error on reading CZTR (SEID=$id) in translation memory" }
    if ($rec[8] =~ /^DETR=(.*)$/) {  } else { die "Error on reading DETR (SEID=$id) in translation memory" }
    if ($rec[9] =~ /^ENTR=(.*)$/) {  } else { die "Error on reading ENTR (SEID=$id) in translation memory" }
    if ($rec[10] !~ /^$/) { die "Error >$rec[7]< on reading separator (SEID=$id) in translation memory" }

    $cz_sentences[$id] = { words => \@czse, chunks => $czch };
    $de_sentences[$id] = { words => \@dese, chunks => $dech };
    $en_sentences[$id] = { words => \@ense, chunks => $ench };
    add_sentence_to_map(\@czse, $czch, \%cz_map, $cz_morpho, $id);
    add_sentence_to_map(\@dese, $dech, \%de_map, $de_morpho, $id);
    add_sentence_to_map(\@ense, $ench, \%en_map, $en_morpho, $id);
  }
  ({ sentences => \@cz_sentences, map => \%cz_map }, { sentences => \@de_sentences, map => \%de_map }, { sentences => \@en_sentences, map => \%en_map })
}


sub add_sentence_to_map
{
  my ($words, $chunks, $map, $morpho_base, $sentence_id) = @_;

  my $chunk_number = 0;
  foreach my $chunk (@$chunks)
  {
    my $occurence = { sentence_id => $sentence_id, chunk_number => $chunk_number };
    foreach my $chunk_part (@$chunk)
    {
      if (looks_like_number($chunk_part))
	{
	  foreach my $lemma (@{ morpho_analyse($morpho_base, $words->[$chunk_part]) })
	  #FIXME: if the lemma is present several times then its occurence is stored the same number of times
	    {
	      push @{$map->{ morpho_get_lemma($lemma) }}, $occurence
	    }
	}
    }
    $chunk_number++;
  }
}


sub parse_chunks
{
  my @result;
  foreach my $chunk (split(/ +/, $_[0]))
  {
    my @word_indices = split(/,/, $chunk);
    foreach my $num (@word_indices) { die "Bad formed alignment (SEID=$_[1]): num: $num" unless ($num =~ /^\d+$/ and --$num < $_[2]) or $num eq '*' }
    #TODO: better input checks - unicity
    push @result, \@word_indices
  }
  \@result
}

sub memory_get_occurences
{
  if (ref($_[0]) eq 'HASH' and defined $_[0]->{map} and ref($_[0]->{map}) eq 'HASH')
  {
    $_[0]->{map}{$_[1]} or die ("memory_get_occurences: $_[1] not in memory")
  }
  else { print "memory_get_occurences: bad type of first argument" }
}

sub memory_get_chunk_elements
{#$memory, $chunk, $debug
  do {
  print "\$_[0]: ", ref($_[0]), "\n";
  print "\$_[0]->{ sentences }: ", ref($_[0]->{ sentences }), "\n";
  print "\$_[0]->{ sentences } [ $_[1]->{sentence_id} ]: ", ref($_[0]->{ sentences } [ $_[1]->{sentence_id} ]), "\n";
  print "\$_[0]->{ sentences } [ $_[1]->{sentence_id} ]  { chunks }: ", ref($_[0]->{ sentences } [ $_[1]->{sentence_id} ] { chunks }), "\n";
  } if $_[2];
  $_[0]->{ sentences } [ $_[1]->{sentence_id} ] { chunks } [ $_[1]->{chunk_number} ]
}

sub my_ref
{
  defined $_[0] ? ref($_[0]) : 'undef'
}

sub memory_get_sentence
{#$memory, $chunk
  print $log "memory_get_sentence: arguments = ", my_ref($_[0]), ', ', my_ref($_[1]), "\n";
  $_[0]->{ sentences } [ $_[1]->{sentence_id} ]
}


sub memory_chunk_get_words
{#$memory, $chunk
  my $sentence = &memory_get_sentence;
  join(' ', map { $sentence->{words}[$_] } @{ &memory_get_chunk_elements })
}

sub transfer
{
  my ($memory, $occurence) = @_;
  my $chunks_sentence = memory_get_sentence($memory, $occurence);
  my @words = map { $chunks_sentence->{words}[$_] } grep(looks_like_number($_) , @{ memory_get_chunk_elements($memory, $occurence) });
  \@words
}

sub memory_chunk_occurence_to_str
{
  "sentence_id: $_[0]->{sentence_id}, chunk_number: $_[0]->{chunk_number}"
}


1
