#!/usr/bin/perl

my $log;
open($log, '>morpho.log');

sub morpho_build
{
  my %morpho_base;
  foreach my $lemma (@{$_[0]})
    {
      foreach my $form (@{$lemma->{forms}})
	{
	  push @{$morpho_base{$form->{form}}}, $lemma;
	  expand_tags($form);
	}
    }
  \%morpho_base
}

{
  my @expanders = (\&expand_P_case, \&expand_NA_case, \&expand_NV_hajic, \&expand_V_person, \&expand_V_number, \&expand_verbal_B, \&expand_VWQ);

  sub expand_tags
    {
      my $form = shift;
      my @queue = @{$form->{tags}};

      foreach my $expander (@expanders)
	{
	  @queue = map { &$expander($_) } @queue
	}
      $form->{etags} = \@queue
    }
}

sub expand_item
{
  my @result;
  #print "EXPAND_ITEM: ", join('---', @_), "\n";
  foreach my $asterisk ($_[1] eq '*' ? @{$_[3]} : split(/,/, $_[1]))
  {
    push @result, $_[0] . $asterisk . $_[2];
  }
  @result
}

sub expand_P_case
{
  if ($_[0] =~ /^(P)((?:(?:[1-7],)+[1-7])|\*)$/)
    {
  #print "expand_P_case\n";
      return expand_item($1, $2, '', [1..7])
    }
  $_[0]
}

sub expand_NA_case
{
  if ($_[0] =~ /^([NA].)((?:(?:[1-7],)+[1-7])|\*)$/)
    {
  #print "expand_NA_case\n";
      return expand_item($1, $2, $3, [1..7])
    }
  $_[0]
}

sub expand_NV_hajic
{
  #print "expand_NV_hajic\n";
  if ($_[0] =~ /^([NV]..)(H)(.*)$/)
    {
      return expand_item($1, '*', $3, ['F', 'N'])
    }

  if ($_[0] =~ /^([NV]..)(Y)(.*)$/)
    {
      return expand_item($1, '*', $3, ['M', 'I'])
    }

  if ($_[0] =~ /^(N..)(Z)$/)
    {
      return expand_item($1, '*', '', ['N', 'M', 'I'])
    }

  if ($_[0] =~ /^(V..)(T)(...)$/)
    {
      return expand_item($1, '*', $3, ['I', 'F'])
    }

  $_[0]
}

sub expand_V_person
{
  #print "expand_V_person\n";
  if ($_[0] =~ /^(V.)((?:(?:[1-3],)+[1-3])|\*)(.*)$/)
    {
      return expand_item($1, $2, $3, [1..3])
    }
  $_[0]
}

sub expand_V_number
{
  #print "expand_V_number\n";
  if ($_[0] =~ /^(V)((?:(?:[SP],)+[SP])|\*)(.*)$/)
    {
      return expand_item($1, $2, $3, ['S', 'P'])
    }
  $_[0]
}

sub expand_verbal_B
{
  #print "expand_verbal_B\n";
  if ($_[0] =~ /^(V...).B(.)$/ )
    {
      return ("$1FA$2", "$1ZA$2")
    }
  $_[0]
}

sub expand_VWQ
{
  #print "expand_VWQ\n";
  if ($_[0] =~ /^VW(.)Q(...)$/ )
    {
      return ("VS$1F$2", "VP$1N$2")
    }
  $_[0]
}


sub morpho_analyse_look_up
{
  print $log "\t'", $_[1], "'", exists $_[0]->{ $_[1] } ? '': ' not', " found\n";
  $_[0]->{ $_[1] }
}


sub morpho_analyse
{
  print $log "morpho_analyse: question='", $_[1], "'\n";
  &morpho_analyse_look_up or morpho_analyse_look_up($_[0], lc $_[1]) or [ { lemma => $_[1], forms => [ { form => $_[1], tags => [ 'X' ] } ] } ]
}

sub morpho_analyse_get_lemmas
{
  my @lemmas = map { morpho_get_lemma($_) } @{ morpho_analyse($_[0], $_[1]) };
  \@lemmas
}

sub morpho_get_lemma
{
  print $log "morpho_get_lemma: argument type: REF=", ref($_[0]), "\n";
  $_[0]->{lemma}
}

sub morpho_get_forms
{
  $_[0]->{forms}
}

sub morpho_get_etags
{
  $_[0]->{etags}
}

sub test
{
  foreach my $lemma (@{morpho_analyse(morpho_build(czech_morpho_base_get()), 'zvýšit')})
    {
      print morpho_get_lemma($lemma), "\n";
      foreach my $form (@{morpho_get_forms($lemma)})
	{
	  print $form->{form}, "\n";
	  foreach my $tag (@{morpho_get_etags($form)}) { print $tag, "\n" }
	}
    }
}


